<?php
/*
Plugin Name: GOP Points widgets
Description: Displays GOP Points information widgets
Version: 1.0
Author: VM Technologies
Author URI: http://www.vmtechnologies.net
*/

/*  Copyright 2010 VM Technologies (email : Support@Vmtechnologies.netm)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

define('GOPPOINTS_REST_ENDPOINT', 'http://developers.gop.com/api/');

 class GOP_Points_Widget extends WP_Widget {
	function GOP_Points_Widget() {
		$widget_ops = array('classname'=>'gop gop_points','description' => __('Your GOP points'));
		$this->WP_Widget('gop_points', __('Your GOP points'), $widget_ops);
	}

	function widget($args, $instance) {
		extract($args);
		$points = goppoints_get_points();
		if ($points===false) return;

		$title = apply_filters('widget_title', esc_attr($instance['title']));
		echo $before_widget.$before_title.$title.$after_title;

		echo '<p>'.$points.'</p>';

		echo $after_widget;
	}

	function update($new_instance, $old_instance) {
		if (!isset($new_instance['submit'])) {
			return false;
		}
		$instance = $old_instance;
		$instance['title'] = strip_tags($new_instance['title']);
		return $instance;
	}

	function form($instance) {
		global $wpdb;
		$instance = wp_parse_args((array) $instance, array('title' => 'Your GOP points'));
		$title = esc_attr($instance['title']);
		?>
		<p>
			<label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:'); ?> <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo $title; ?>" /></label>
		</p>
		<input type="hidden" id="<?php echo $this->get_field_id('submit'); ?>" name="<?php echo $this->get_field_name('submit'); ?>" value="1" />
		<?php
	}
}

 class GOP_Rank_Widget extends WP_Widget {
	function GOP_Rank_Widget() {
		$widget_ops = array('classname'=>'gop gop_rank','description' => __('Your GOP rank'));
		$this->WP_Widget('gop_rank', __('Your GOP rank'), $widget_ops);
	}

	function widget($args, $instance) {
		extract($args);

		$ranktotal = goppoints_get_rank($instance['locale']);
		if ($ranktotal===false) return;

		$title = apply_filters('widget_title', esc_attr($instance['title']));
		echo $before_widget.$before_title.$title.$after_title;

		echo '<p><span class="gop_rank_1">'.$ranktotal->rank.'</span><span class="gop_rank_2"> out of <span class="gop_rank_3">'.$ranktotal->total.'</span></span></p>';

		echo $after_widget;
	}

	function update($new_instance, $old_instance) {
		if (!isset($new_instance['submit'])) {
			return false;
		}
		$instance = $old_instance;
		$instance['title'] = strip_tags($new_instance['title']);
		$instance['locale'] = strip_tags($new_instance['locale']);

		return $instance;
	}

	function form($instance) {
		global $wpdb;
		$instance = wp_parse_args((array) $instance, array('title' => 'Your GOP rank'));
		$title = esc_attr($instance['title']);
		$locale = esc_attr($instance['locale']);
		?>
		<p>
			<label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:'); ?> <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo $title; ?>" /></label>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('locale'); ?>"><?php _e('Locale:'); ?> <input class="widefat" id="<?php echo $this->get_field_id('locale'); ?>" name="<?php echo $this->get_field_name('locale'); ?>" type="text" value="<?php echo $locale; ?>" /></label><br/><small>5 digit zip code or 2 character state abbreviation (optional)</small>
		</p>
		<input type="hidden" id="<?php echo $this->get_field_id('submit'); ?>" name="<?php echo $this->get_field_name('submit'); ?>" value="1" />
		<?php
	}
}

 class GOP_Badge_Widget extends WP_Widget {
	function GOP_Badge_Widget() {
		$widget_ops = array('classname'=>'gop gop_badge','description' => __('Your GOP badge'));
		$this->WP_Widget('gop_badge', __('Your GOP badge'), $widget_ops);
	}

	function widget($args, $instance) {
		extract($args);

		$badge = goppoints_get_badge();
		if ($badge===false) return;

		$title = apply_filters('widget_title', esc_attr($instance['title']));
		echo $before_widget.$before_title.$title.$after_title;

		echo '<script type="text/javascript" src="'.$badge.'" /></script>';

		echo $after_widget;
	}

	function update($new_instance, $old_instance) {
		if (!isset($new_instance['submit'])) {
			return false;
		}
		$instance = $old_instance;
		$instance['title'] = strip_tags($new_instance['title']);

		return $instance;
	}

	function form($instance) {
		global $wpdb;
		$instance = wp_parse_args((array) $instance, array('title' => 'Your GOP badge'));
		$title = esc_attr($instance['title']);
		?>
		<p>
			<label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:'); ?> <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo $title; ?>" /></label>
		</p>
		<input type="hidden" id="<?php echo $this->get_field_id('submit'); ?>" name="<?php echo $this->get_field_name('submit'); ?>" value="1" />
		<?php
	}
}

 class GOP_Leaders_Widget extends WP_Widget {
	function GOP_Leaders_Widget() {
		$widget_ops = array('classname'=>'gop gop_leaders','description' => __('GOP leaderboard'));
		$this->WP_Widget('gop_leaders', __('GOP leaderboard'), $widget_ops);
	}

	function widget($args, $instance) {
		extract($args);

	    $leaderboard = goppoints_get_leaders($instance['locale'],$instance['limit']);

        if ($leaderboard===false) return;

		$title = apply_filters('widget_title', esc_attr($instance['title']));

		echo $before_widget.$before_title.$title.$after_title;

		echo '<ol>';
		foreach ($leaderboard->user as $next) {
			echo '<li><span class="gop_leaders_1">'.$next->username.':</span> <span class="gop_leaders_2">'.$next->points.'</span></li>';
		}
		echo '</ol>';



		echo $after_widget;
	}

	function update($new_instance, $old_instance) {
		if (!isset($new_instance['submit'])) {
			return false;
		}
		$instance = $old_instance;
		$instance['title'] = strip_tags($new_instance['title']);
		$instance['locale'] = strip_tags($new_instance['locale']);
		$instance['limit'] = intval($new_instance['limit']);
		if ($instance['limit']<1) $instance['limit']=1;
		if ($instance['limit']>100) $instance['limit']=100;

		return $instance;
	}

	function form($instance) {
		global $wpdb;
		$instance = wp_parse_args((array) $instance, array('title' => 'GOP leaderboard'));
		$title = esc_attr($instance['title']);
		$locale = esc_attr($instance['locale']);
		$limit = esc_attr($instance['limit']);
		if (!$limit) $limit = 10;
		?>
		<p>
			<label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:'); ?> <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo $title; ?>" /></label>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('locale'); ?>"><?php _e('Locale:'); ?> <input class="widefat" id="<?php echo $this->get_field_id('locale'); ?>" name="<?php echo $this->get_field_name('locale'); ?>" type="text" value="<?php echo $locale; ?>" /></label><br/><small>5 digit zip code or 2 character state abbreviation (optional)</small>
		</p>
		<p>
			<label for="<?php echo $this->get_field_id('limit'); ?>"><?php _e('Number of results:'); ?> <input class="widefat" id="<?php echo $this->get_field_id('limit'); ?>" name="<?php echo $this->get_field_name('limit'); ?>" type="text" value="<?php echo $limit; ?>" /></label><br/><small>Number of results to show on leaderboard (1-100)</small>
		</p>
		<input type="hidden" id="<?php echo $this->get_field_id('submit'); ?>" name="<?php echo $this->get_field_name('submit'); ?>" value="1" />
		<?php
	}
}

add_action('widgets_init', 'widget_goppoints_init');
function widget_goppoints_init() {
	register_widget('GOP_Points_Widget');
	register_widget('GOP_Rank_Widget');
	register_widget('GOP_Badge_Widget');
	register_widget('GOP_Leaders_Widget');
}


add_action('admin_menu','goppoints_menu');

function goppoints_menu() {
	add_options_page('GOP Points options','GOP Points options','administrator','goppoints_options_page','goppoints_options_page');
}

function goppoints_options_page() {
	$email = get_option('goppoints_email');
	$cache = get_option('goppoints_cache_duration');

	if ($_POST['goppoints_submit']) {
		if ($_POST['goppoints_email']) $email = $_POST['goppoints_email'];
		if ($_POST['goppoints_cache_duration']) $cache = $_POST['goppoints_cache_duration'];

		update_option('goppoints_email',$email);
		update_option('goppoints_cache_duration',$cache);
	}

	?>
	<div class="wrap">
	<h2>GOP Points settings</h2>


	<form name="goppoints_settings" method="post" action="">
		<table class="form-table">
		<tr valign="top">

		<th scope="row"><label for="goppoints_email">Email address:</label></th>
		<td><input name="goppoints_email" type="text" id="goppoints_email" value="<?php echo esc_attr($email);?>" class="regular-text" />
		<span class="description">Your GOP Points email address</code></span>
		</td>
		</tr>

  		<?php $times = array(900=>'15 min', 1800=>'30 min', 2700=>'45 min', 3600=>'1 hour', 7200=>'2 hours', 10800=>'3 hours', 14400=>'4 hours', 18000=>'5 hours', 21600=>'6 hours', 43200=>'12 hours', 86400=>'1 day'); ?>

		<tr valign="top">
		<th scope="row"><label for="goppoints_cache_duration">Update interval</label></th>
		<td><select name="goppoints_cache_duration"><?php foreach ($times as $key=>$val) { ?><option value="<?php echo $key;?>"<?php if ($key==$cache) echo ' selected="selected"';?>><?php echo $val;?></option><?php } ?></select>
		<span class="description">The refresh interval indicating how often you want to check cached GOP Points API calls are up to date.</span>
		</td>
		</tr>

		</table>

		<input type="hidden" name="goppoints_submit" value="1" />

		<p class="submit"><input class="button-primary" type="submit" value="Save &raquo;" /></p>
	</form>

	</div>
	<?php
}

function goppoints_request($method) {

  $url = GOPPOINTS_REST_ENDPOINT . $method;

  $cache = get_option("goppoints_cache");

  if (!$cache) $cache = array();

  foreach ($cache as $key=>$next) {
  	if ($next[0]<time()) unset($cache[$key]);
  }
  if ($cache["goppoints_$method"]) {
	return simplexml_load_string($cache["goppoints_$method"][1]);
  }


  // If a cached value wasn't suitable, attempt to connect and fetch a result.

  $ch = curl_init($url);
  curl_setopt($ch,CURLOPT_HEADER,false);
  curl_setopt($ch,CURLOPT_RETURNTRANSFER,true);
  $result = curl_exec($ch);
  $status = curl_getinfo($ch,CURLINFO_HTTP_CODE);

  $ret = FALSE;

  if ($status==200) {
  	$response = simplexml_load_string($result);
  	if ($response) {
  		if (!goppoints_response_has_error($response)) {
  			$ret = $response;
  		}
  	}
  }

  $duration = get_option('goppoints_cache_duration');
  if (!$duration) $duration = 900;

  $cache["goppoints_$method"][0] = time()+$duration;
  if ($ret!==false) $cache["goppoints_$method"][1] = $result;

  update_option("goppoints_cache",$cache);

  if ($ret) return $response;
  else if ($cache["goppoints_$method"][1]) return simplexml_load_string($cache["goppoints_$method"][1]);
  else return FALSE;
}

function goppoints_response_has_error($response) {
  return isset($response->statuscode);
}

function goppoints_get_points() {
  $email = get_option('goppoints_email');

  if (!$email) return FALSE;

  $response = goppoints_request("get/points/user/".urlencode($email)."/");

  if ($response) {
    return $response->user->points;
  }
  return FALSE;
}

function goppoints_get_leaders($locale='',$limit=0) {
  $args = array();
  if ($locale) $args['locale'] = 'locale='.urlencode($locale);
  if ($limit) $args['limit'] = 'limit='.$limit;
  $method = "get/points/leaders/";
  if ($args) $method.='?'.join("&",$args);
  $response = goppoints_request($method);
  if ($response) {
    return $response;
  }
  return FALSE;
}

function goppoints_get_rank($locale='') {
  $email = get_option('goppoints_email');
  if (!$email) return FALSE;
  $method = "get/points/rank/".urlencode($email)."/";
  if ($locale) $method.="?locale=".urlencode($locale);

  $response = goppoints_request($method);
  if ($response) {
    return $response->user;
  }
  return FALSE;
}

function goppoints_get_badge() {
  $email = get_option('goppoints_email');
  if (!$email) return FALSE;
  return GOPPOINTS_REST_ENDPOINT."get/points/badge/".urlencode($email)."/";
}